#include <acknex.h>
#include "square.h"
#include "block.h"
#include "gameField.h"
#include "gameLevel.h"

//////////////////////////////////////////////////////////
/////////////// MAIN GAME ENGINE
/////////////////////////////////////////////////////////

// Next Block
Block *nextBlock;

// CurrentBlock
Block *currentBlock;

// Score
var iScore = 0;
// Next level score
var iLastLevelScore = 0;
// Score to level up
var iLevelScorePoint = 5000;


// level
var iLevel = 0;

// Game over
BOOL bGameOver = FALSE;

// Game Start
BOOL bStart = TRUE;

// Paused
BOOL bPaused = FALSE;

// Level timer
float dLevelTimer;

// Font
FONT *arial_font = "Arial#30b";
FONT *courier_font = "Courier#20b";
FONT *couriern_font = "Courier#12";

// Chaines
STRING *str_title = "QuadTrix";
STRING *str_next = "Next : ";

TEXT *titreText = 
{
	font = arial_font;
	string(str_title);
	flags = SHOW;
}

TEXT *titreText2 = 
{
	pos_x = 130;
	pos_y = 10; 
	font = couriern_font;
	string("-= Simple Tetris clone made with lite-C(quads).=-");
	flags = SHOW;
}

TEXT *nextText = 
{
	pos_x = 315;
	pos_y = 100;
	font = courier_font;
	string(str_next);
	flags = SHOW;
}

PANEL *panScore = 
{
	pos_x = 315;
	pos_y = 23;
	digits(0, 30, "Score : %.0f", courier_font, 1, iScore);
	flags = SHOW;
}

PANEL *panLevel = 
{
	pos_x = 315;
	pos_y = 210;
	digits(0, 30, "Level : %.0f", courier_font, 1, iLevel);
	flags = SHOW;
}

TEXT *panCommand = 
{
	pos_x = 315;
	pos_y = 290;
	font = couriern_font;
	string("Command : \n1 - UP : Turn block\n2 - DOWN : Move block down\n3 - LEFT : Move block left\n4 - RIGHT : Move block right\n5 - R : Reset game\n6 - P : Pause/Resume game.");
	flags = SHOW;
}

TEXT *panNote = 
{
	pos_x = 315;
	pos_y = 390;
	font = couriern_font;
	string("Note : \nThis game(demo) is for demonstration purpose only,\nso there may be some bugs! :)");
	flags = SHOW;
}


/**
 * Changement de niveau
 */
void game_setLevel(int pLevel)
{
	int level = pLevel;
	if((level <= 0) || level > 10)
		level = 1;
	// Augmentation de niveau
	iLevel = level;
	
	// Calculer le timer
	dLevelTimer = 1 / iLevel;
	
	// Score
	iLastLevelScore = iScore;
}

/**
 * Rinitialiser le jeu
 */
void game_reset()
{
	// Initialisation des variables
	bStart = TRUE;
	bPaused = FALSE;
	bGameOver = FALSE;
	iScore = 0;
	currentBlock = NULL;
	nextBlock = NULL;
	
	// Level
	game_setLevel(1);
	
	//Initialisation du plateau de jeu
	int i = 0, j = 0;
	for(i = 0; i<FIELD_WIDTH; i++)
	{
		// Initialiser les bits
		arrBitGameField[i] = BIT_EMPTY;
		// Initialiser les carres
		for(j = 0; j<FIELD_HEIGHT; j++)
		{
			arrSquareField[i][j] = NULL;
		}
	}
}

/**
 * Pauser le jeu
 */
void game_pause()
{
	bPaused = !bPaused;
}

/**
 * Game control
 */
void key_control()
{
	// wait for block
	while(currentBlock == NULL)
	{
		wait(1);
	}
	// control loop
	while(1)
	{
		if(key_cuu && !bPaused)
		{
			block_rotate(currentBlock);
			wait(-.2);
		}
		else if(key_cud && !bPaused)
		{
			block_down(currentBlock);
			// rduction de l'attente
			wait(1);	
		}
		else if(key_cul && !bPaused)
		{
			block_left(currentBlock);
			wait(-.2);
		}
		else if(key_cur && !bPaused)
		{
			block_right(currentBlock);
			wait(-.2);
		}
		else if(key_r)
		{
			// rinitialiser le jeu
			game_reset();
			wait(-.2);
		}
		else if(key_p)
		{
			game_pause();
			wait(-.2);
		}
		// Attente frame
		wait(1);
	}
}

/**
 * Game timerControl
 */
void timer_control()
{	
	while( (currentBlock == NULL) && (!bStart))
	{
		wait(1);
	}
	// Block down loop
	while(1)
	{
			// Attendre en cas de pause
		while(bPaused)
		{
			wait(1);
		}
			// Aucyn traitement si gameOver
		if(!bGameOver) {
			// Crer le premier bloc
			if(bStart)
			{
				// Crer un bloc alatoire
				currentBlock = block_new(vector(SQUARE_SIZE * 6, 0, 0), 0);
				
				// Crer un block alatoire pour le prochain bloc
				nextBlock = block_new(vector(SQUARE_SIZE * 20, SQUARE_SIZE * 7, 0), random(8));
				bStart = FALSE;
			}
			
			// Test for block down
			if(!block_down(currentBlock))
			{
				// Test de game Over
				if( block_isTop(currentBlock) )
				{
					// Game over
					bGameOver = true;
					printf("Game Over.\nPress R to play again.");
				}
				// Le bloc est stopp; crer un autre bloc
				else
				{
					// Nb de lignes effaces
					var erasedLines = gameField_checkLines();
					// Point
					iScore += 100 * erasedLines;
					
					// Test for level up
					if((iScore - iLastLevelScore) >= iLevelScorePoint)
					{
						iLevel ++;
						game_setLevel(iLevel);
					}
					
					
					// Remplacer le bloc en cours par le bloc suivant et crer le bloc suivant
					currentBlock = block_new(vector(SQUARE_SIZE * 6, 0, 0), nextBlock->m_Type);
					// Librerla mmoire occup par le bloc prcdent
					free(nextBlock);
					nextBlock = block_new(vector(SQUARE_SIZE * 20, SQUARE_SIZE * 7, 0), 0);
				}
			}
		}
		wait(-dLevelTimer);
	}
}

/**
 * Main function
 */
void main()
{
	level_load("");
	// wait for video device
	wait(2);
	
	vec_set(sky_color, vector(0x99, 0x99, 0x99));

	// logo to bottom left
	//logo = 3;
	
	// Niveau
	game_setLevel(iLevel);
	
	// Initialisation gameField
	gameField_init();
	
	// game control
	ent_create("", nullvector, key_control);
	// timer control
	ent_create("", nullvector, timer_control);
		
	// Main loop
	while(1)
	{
		// dessiner le niveau
		gameLevel_draw();
		
		// dessiner le bloc en cours
		block_draw(currentBlock);
		
		// dessiner le bloc en suivant
		block_draw(nextBlock);
		
		// dessiner le carres sur le terrai de jeu
		gameField_drawSquares();
		
		// Dessiner le masque de pause
		if(bPaused)
			gameLevel_drawPaused();
		
		// wait 1 frame
		wait(1);
	}
}