// Variables de jeu

#ifndef __GAME_FIELD_H__
#define __GAME__FIELD_H__

// Tailles du jeu
#define FIELD_WIDTH	16
#define FIELD_HEIGHT	30
#define SQUARE_SIZE	15

#define START_X	SQUARE_SIZE * 3
#define START_Y	SQUARE_SIZE * 4

// Masques des cases
#define BIT_EMPTY	0x00000000
#define BIT_FULL	0x0001FFFE

// Tabeau de jeux
int arrBitGameField[FIELD_HEIGHT];

// Tableau des squares
Square *arrSquareField[FIELD_WIDTH][FIELD_HEIGHT];

// Couleurs des blocs
VECTOR *blockColors[7][2];

// Emplacement vide
BOOL gameField_isEmpty(int x, int y)
{
	BOOL empty = TRUE;
		
	// Si l'emplacement est au dessus de la table de jeu renvoyer faux
	if ((y < 0 || y >= FIELD_HEIGHT) || (x < 0 || x >= FIELD_WIDTH))
	{
		empty = FALSE;	
	}
	// Tester le bit  la position
	else if (arrBitGameField[y] & (2 << x))
	{
		if (arrSquareField[x][y] != NULL)
			empty = FALSE;
	}
	return empty;
}

// Rechercher les lignes remplies
int gameField_checkLines()
{
	int x = 0, y = 0;		// ligne & colonne
	int i = 0;			// compteur de Boucle
	int nbLignes = 0;	// Nombres de lignes effaces
	// suppression de la ligne
	BOOL bModif = TRUE;
	
	y = FIELD_HEIGHT - 1;
	while(bModif)
	{
		
		// Si on arrive  une ligne vide, arreter
		if(arrBitGameField[y] == BIT_EMPTY)
		{
			// Rinitialiser la modification
			bModif = FALSE;
		}
				
		// Si la ligne est remplie; incrmentaert le nombre de lignes
		// et descendre les lignes en haut
		else if(arrBitGameField[y] == BIT_FULL)
		{
			nbLignes ++;
				
			// Dplacer les lignes vers le bas
			for(i = y; i>=0; i--) {
				// Si la ligne n'est pas la premire dans le jeu
				// copier la ligne au dessus
				if(i > 0) {
					// Copier les bits
					arrBitGameField[i] = arrBitGameField[i - 1];
					
					// Copie des squares
					for(x = 0; x < FIELD_WIDTH; x++) {
						arrSquareField[x][i] = arrSquareField[x][i-1];
						// Modification des places des carres
						if (arrSquareField[x][i] != NULL){
							Square *square = (Square*) arrSquareField[x][i];
							square->m_Location.y += SQUARE_SIZE;
						}
					}
				}
				// Sinon effacer la ligne
				else {
					arrBitGameField[i] = BIT_EMPTY;
					for(x = 0; x < FIELD_WIDTH; x++) {
						arrSquareField[x][i] = NULL;
					}
				}
			}
			
			// Modification
			bModif = TRUE;
		}
		// Non vide !=BIT_FULL && !=BIT_EMPTY
		else //(arrBitGameField[y] > 0)
		{
			// Dcrmenter y
			y --;
			// Modification
			bModif = TRUE;
		}
		
	}
	
	// renvoyer le nombre de lignes
	return nbLignes;
}

// Stopper les squares
void gameField_stopSquare(Square *square, int x, int y)
{	
	// set bit
	int shift = 2 << x;
	int k = (arrBitGameField[y] | (shift));
	arrBitGameField[y] = k;
	
	//printf("%d, %d", x, y);
	// set square
	arrSquareField[x][y] = square;
}

// Draw game
void gameField_drawSquares()
{
	int x; int y;
	
	// Dessiner les squares j'usqu'a la ligne vide
	y = FIELD_HEIGHT - 1;
	// Parcourir les lignes
	while(( y >= 0) && ( arrBitGameField[y] != BIT_EMPTY ))
	{
		
		for(x = FIELD_WIDTH - 1; x>=0; x--)
		{
			if(arrSquareField[x][y] != NULL)
			{
				Square *square = arrSquareField[x][y];
				square_draw(square);
			}
		}
		
		// dcrmenter y
		y --;
	}
	
}

// square au dessus du plateau::Game over
BOOL gameField_isSquareTop(int y)
{
	BOOL top = FALSE;
	if (y <= 0)
	{
		top = TRUE;
	}
	return top;
}

// Initialisation des couleurs
void gameField_initBlockColors()
{
	int i = 0, j = 0;
	int n = 7, m = 2;
	
	// Initialisation des vecteurs
	for(i = 0; i<n; i++) {
		for(j = 0; j<m; j++)
		{
			// Initialisation bloc
			blockColors[i][j] = (VECTOR *)malloc(sizeof(VECTOR));
			vec_set(blockColors[i][j], nullvector);
		}
	}
	
	// Affectation des couleurs
	vec_set(blockColors[0][0], vector(0xff, 0x00, 0x00)); vec_set(blockColors[0][1], vector(0xff, 0x66, 0x33));
	vec_set(blockColors[1][0], vector(0x00, 0x00, 0xff)); vec_set(blockColors[1][1], vector(0x33, 0x66, 0xff));
	vec_set(blockColors[2][0], vector(0x02, 0xc4, 0xf4)); vec_set(blockColors[2][1], vector(0x8c, 0xe1, 0xf7));
	vec_set(blockColors[3][0], vector(0xdf, 0x8d, 0xee)); vec_set(blockColors[3][1], vector(0xf3, 0xbb, 0xfa));
	vec_set(blockColors[4][0], vector(0x02, 0xe2, 0xe2)); vec_set(blockColors[4][1], vector(0x37, 0xfe, 0xfe));
	vec_set(blockColors[5][0], vector(0x00, 0xee, 0x00)); vec_set(blockColors[5][1], vector(0x66, 0xff, 0x66));
	vec_set(blockColors[6][0], vector(0xcc, 0xcc, 0x00)); vec_set(blockColors[6][1], vector(0xfd, 0xfd, 0x1c));
}

// Initialisation du jeu
void gameField_init()
{
	// Couleur des blocs
	gameField_initBlockColors();
}

// __GAME_FIELD_H__
#endif